package com.ibm.websphere.samples.plantsbywebsphereejb;


import java.util.Collection;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.TableGenerator;


/**
 * Bean implementation class for Enterprise Bean: Order
 */
@Entity(name="Order")
@Table(name="ORDER1", schema="APP")
@NamedQueries({
	@NamedQuery(
		name="removeAllOrders",
		query="delete from Order")
})
@SuppressWarnings("unchecked")
public class Order
{
	public static final String ORDER_INFO_TABLE_NAME = "java:comp/env/jdbc/OrderInfoTableName";
	public static final String ORDER_ITEMS_TABLE_NAME = "java:comp/env/jdbc/OrderItemsTableName";
	
	@Id
	@GeneratedValue(strategy=GenerationType.TABLE,generator="OrderSeq")
	@TableGenerator(name="OrderSeq", table="IDGENERATOR", pkColumnName="IDNAME", 
			pkColumnValue="ORDER", valueColumnName="IDVALUE")
	private String orderID;
	private String sellDate;
	private String billName;
	private String billAddr1;
	private String billAddr2;
	private String billCity;
	private String billState;
	private String billZip;
	private String billPhone;
	private String shipName;
	private String shipAddr1;
	private String shipAddr2;
	private String shipCity;
	private String shipState;
	private String shipZip;
	private String shipPhone;
	private String creditCard;
	private String ccNum;
	private String ccExpireMonth;
	private String ccExpireYear;
	private String cardHolder;
	private int shippingMethod;
	private float profit;
	
	@ManyToOne
	@JoinColumn(name="CUSTOMERID")
	private Customer customer;
	private Collection orderItems;
	
	private Collection items = null;
	
	/**
	 * Constructor to create an Order.
	 *
	 * @param customer - customer who created the order 
	 * @param billName - billing name
	 * @param billAddr1 - billing address line 1
	 * @param billAddr2 - billing address line 2
	 * @param billCity - billing address city
	 * @param billState - billing address state
	 * @param billZip - billing address zip code
	 * @param billPhone - billing phone
	 * @param shipName - shippng name
	 * @param shipAddr1 - shippng address line 1
	 * @param shipAddr2 - shippng address line 2
	 * @param shipCity - shippng address city
	 * @param shipState - shippng address state
	 * @param shipZip - shippng address zip code
	 * @param shipPhone - shippng phone
	 * @param creditCard - credit card
	 * @param ccNum - credit card number
	 * @param ccExpireMonth - credit card expiration month
	 * @param ccExpireYear - credit card expiration year
	 * @param cardHolder - credit card holder name
	 * @param shippingMethod int of shipping method used
	 * @param items vector of StoreItems ordered
	 */
	public Order(Customer customer, String billName, String billAddr1, String billAddr2,
			String billCity, String billState, String billZip, String billPhone,
			String shipName, String shipAddr1, String shipAddr2,
			String shipCity, String shipState, String shipZip, String shipPhone,
			String creditCard, String ccNum, String ccExpireMonth, String ccExpireYear,
			String cardHolder, int shippingMethod, Collection items)
	{
		this.setSellDate(Long.toString(System.currentTimeMillis()));
		
		// Pad it to 14 digits so sorting works properly.
		if (this.getSellDate().length() < 14)
		{
			StringBuffer sb = new StringBuffer(Util.ZERO_14);
			sb.replace((14 - this.getSellDate().length()), 14, this.getSellDate());
			this.setSellDate(sb.toString());
		}
		
		this.setCustomer(customer);
		this.setBillName(billName);
		this.setBillAddr1(billAddr1);
		this.setBillAddr2(billAddr2);
		this.setBillCity(billCity);
		this.setBillState(billState);
		this.setBillZip(billZip);
		this.setBillPhone(billPhone);
		this.setShipName(shipName);
		this.setShipAddr1(shipAddr1);
		this.setShipAddr2(shipAddr2);
		this.setShipCity(shipCity);
		this.setShipState(shipState);
		this.setShipZip(shipZip);
		this.setShipPhone(shipPhone);
		this.setCreditCard(creditCard);
		this.setCcNum(ccNum);
		this.setCcExpireMonth(ccExpireMonth);
		this.setCcExpireYear(ccExpireYear);
		this.setCardHolder(cardHolder);
		this.setShippingMethod(shippingMethod);
		this.items = items;
		
		// Get profit for total order.
		OrderItem oi;
		float profit;
		profit = 0.0f;
		for (Object o : items)
		{
			oi = (OrderItem) o;
			profit = profit + (oi.getQuantity() * (oi.getPrice() - oi.getCost()));
			oi.setOrder(this);
		}
		this.setProfit(profit);
		/*
		 IdGeneratorHome idGeneratorHome = (IdGeneratorHome) Util.getEJBLocalHome("java:comp/env/ejb/IdGenerator",
		 com.ibm.websphere.samples.plantsbywebsphereejb.IdGeneratorHome.class);
		 
		 try
		 {        
		 IdGenerator idGenerator= idGeneratorHome.findByPrimaryKeyUpdate("ORDER");
		 int orderInt = idGenerator.nextId();
		 this.setOrderID(Integer.toString(orderInt)); 
		 }
		 catch(Exception e) 
		 {        
		 e.printStackTrace();
		 throw new RuntimeException();
		 }
		 */
		
		
	}
	public Order(String orderID)
	{
		setOrderID(orderID);
	}
	public Order() { }
	
	public String getBillAddr1() {
		return billAddr1;
	}
	
	public void setBillAddr1(String billAddr1) {
		this.billAddr1 = billAddr1;
	}
	
	public String getBillAddr2() {
		return billAddr2;
	}
	
	public void setBillAddr2(String billAddr2) {
		this.billAddr2 = billAddr2;
	}
	
	public String getBillCity() {
		return billCity;
	}
	
	public void setBillCity(String billCity) {
		this.billCity = billCity;
	}
	
	public String getBillName() {
		return billName;
	}
	
	public void setBillName(String billName) {
		this.billName = billName;
	}
	
	public String getBillPhone() {
		return billPhone;
	}
	
	public void setBillPhone(String billPhone) {
		this.billPhone = billPhone;
	}
	
	public String getBillState() {
		return billState;
	}
	
	public void setBillState(String billState) {
		this.billState = billState;
	}
	
	public String getBillZip() {
		return billZip;
	}
	
	public void setBillZip(String billZip) {
		this.billZip = billZip;
	}
	
	public String getCardHolder() {
		return cardHolder;
	}
	
	public void setCardHolder(String cardHolder) {
		this.cardHolder = cardHolder;
	}
	
	public String getCcExpireMonth() {
		return ccExpireMonth;
	}
	
	public void setCcExpireMonth(String ccExpireMonth) {
		this.ccExpireMonth = ccExpireMonth;
	}
	
	public String getCcExpireYear() {
		return ccExpireYear;
	}
	
	public void setCcExpireYear(String ccExpireYear) {
		this.ccExpireYear = ccExpireYear;
	}
	
	public String getCcNum() {
		return ccNum;
	}
	
	public void setCcNum(String ccNum) {
		this.ccNum = ccNum;
	}
	
	public String getCreditCard() {
		return creditCard;
	}
	
	public void setCreditCard(String creditCard) {
		this.creditCard = creditCard;
	}
	
	public Customer getCustomer() {
		return customer;
	}
	
	public void setCustomer(Customer customer) {
		this.customer = customer;
	}
	
	public Collection getItems() {
		return items;
	}
	
	public void setItems(Collection items) {
		this.items = items;
	}
	
	public String getOrderID() {
		return orderID;
	}
	
	public void setOrderID(String orderID) {
		this.orderID = orderID;
	}
	// TODO: whats the diff!! items != orderitems?
	public Collection getOrderItems() {
		return orderItems;
	}
	
	public void setOrderItems(Collection orderItems) {
		this.orderItems = orderItems;
	}
	
	public float getProfit() {
		return profit;
	}
	
	public void setProfit(float profit) {
		this.profit = profit;
	}
	
	public String getSellDate() {
		return sellDate;
	}
	
	public void setSellDate(String sellDate) {
		this.sellDate = sellDate;
	}
	
	public String getShipAddr1() {
		return shipAddr1;
	}
	
	public void setShipAddr1(String shipAddr1) {
		this.shipAddr1 = shipAddr1;
	}
	
	public String getShipAddr2() {
		return shipAddr2;
	}
	
	public void setShipAddr2(String shipAddr2) {
		this.shipAddr2 = shipAddr2;
	}
	
	public String getShipCity() {
		return shipCity;
	}
	
	public void setShipCity(String shipCity) {
		this.shipCity = shipCity;
	}
	
	public String getShipName() {
		return shipName;
	}
	
	public void setShipName(String shipName) {
		this.shipName = shipName;
	}
	
	public String getShipPhone() {
		return shipPhone;
	}
	
	public void setShipPhone(String shipPhone) {
		this.shipPhone = shipPhone;
	}
	
	public int getShippingMethod() {
		return shippingMethod;
	}
	
	public void setShippingMethod(int shippingMethod) {
		this.shippingMethod = shippingMethod;
	}
	
	public String getShipZip() {
		return shipZip;
	}
	
	public void setShipZip(String shipZip) {
		this.shipZip = shipZip;
	}
	public String getShipState() {
		return shipState;
	}
	public void setShipState(String shipState) {
		this.shipState = shipState;
	}
}
