//
//"This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
//instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
//or for redistribution by customer, as part of such an application, in customer's own products. " 
//
//Product 5630-A36,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
//All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.plantsbywebsphereejb;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.LockModeType;
import javax.persistence.PersistenceContext;

/**
 * ShoppingCartBean is the implementation class for the {@link ShoppingCart} stateful session
 * EJB.  ShoppingCartBean implements each of the business methods in the <code>ShoppingCart</code>
 * EJB local interface and each of the EJB lifecycle methods in the javax.ejb.SessionBean
 * interface.
 * 
 * @see ShoppingCart
 */
@Stateful(name="ShoppingCart")
public class ShoppingCartBean implements ShoppingCart
{

	@PersistenceContext(unitName="PBW")
	EntityManager em;
	
	private ArrayList<ShoppingCartItem> items = new ArrayList<ShoppingCartItem>();
	
	/**
	 * Get the inventory item.
	 *
	 * @param id of inventory item.
	 * @return an inventory bean.
	 */
	private Inventory getInventoryItem(String inventoryID)
	{
		Inventory inv = null;
		/*      // Get the Inventory home.
		 InventoryHome inventoryHome = (InventoryHome) Util.getEJBLocalHome("java:comp/env/ejb/Inventory", InventoryHome.class);
		 try
		 {
		 inv = inventoryHome.findByPrimaryKeyUpdate(inventoryID);
		 }
		 catch (FinderException e)
		 {
		 Util.debug("ShoppingCartBean.getInventoryItem() - Exception: " + e);
		 }
		 */
		inv = em.find(Inventory.class, inventoryID);
		return inv;
	}
	/**
	 * Create a shopping cart.
	 *
	 * @param cartContents Contents to populate cart with.
	 */
	public void setCartContents(ShoppingCartContents cartContents)
	{
		items = new ArrayList<ShoppingCartItem>();
		int qty;
		String inventoryID;
		ShoppingCartItem si;
		Inventory inv;
		for (int i = 0; i < cartContents.size(); i++)
		{
			inventoryID = cartContents.getInventoryID(i);
			qty = cartContents.getQuantity(inventoryID);
			// Get the Inventory home.
			/*
			InventoryHome inventoryHome = (InventoryHome) Util.getEJBLocalHome("java:comp/env/ejb/Inventory", com.ibm.websphere.samples.pbwjpa.InventoryHome.class);
			try
			{ */
			// inv = inventoryHome.findByPrimaryKey(new InventoryKey(inventoryID));
			inv = em.find(Inventory.class, inventoryID);
			// clone so we can use Qty as qty to purchase, not inventory in stock
			si = new ShoppingCartItem(inv);
			si.setQuantity(qty);
			addItem(si);
			/* }
			 */
		}
	}
	
	/** 
	 * Add an item to the cart.
	 *
	 * @param new_item Item to add to the cart.
	 */
	public void addItem(ShoppingCartItem new_item)
	{
		boolean added = false;
		ShoppingCartItem old_item;
		// If the same item is already in the cart, just increase the quantity.
		for (int i = 0; i < items.size(); i++)
		{
			old_item = (ShoppingCartItem) items.get(i);
			if (new_item.equals(old_item))
			{
				old_item.setQuantity(old_item.getQuantity() + new_item.getQuantity());
				added = true;
				break;
			}
		}
		// Add this item to shopping cart, if it is a brand new item.
		if (!added)
			items.add(new_item);
	}
	/** 
	 * Remove an item from the cart.
	 *
	 * @param item Item to remove from cart.
	 */
	public void removeItem(ShoppingCartItem item)
	{
		for (Object obj : items)
		{
			ShoppingCartItem si = (ShoppingCartItem) obj;
			if (item.equals(si))
			{
				items.remove(si);
				break;
			}
		}
	}
	/** 
	 * Get the items in the shopping cart.
	 *
	 * @return A Collection of ShoppingCartItems.
	 */
	public Collection <ShoppingCartItem>getItems()
	{
		return items;
	}
	/** 
	 * Set the items in the shopping cart.
	 *
	 * @param items A Vector of ShoppingCartItem's.
	 */
	@SuppressWarnings("unchecked")
	public void setItems(Collection items)
	{
		this.items = new ArrayList<ShoppingCartItem>(items);
	}
	/** 
	 * Get the total cost of all items in the shopping cart.
	 *
	 * @return The total cost of all items in the shopping cart.
	 */
	public float getTotalCost()
	{
		float total = 0.0f;
		ShoppingCartItem si;
		for (Object o : items)
		{
			si = (ShoppingCartItem) o;
			total += si.getPrice() * si.getQuantity();
		}
		return total;
	}
	/** 
	 * Method checkInventory.
	 * Check the inventory level of a store item.
	 * Order additional inventory when necessary.
	 *
	 * @param si - Store item
	 */
	public void checkInventory(ShoppingCartItem si)
	{
		Util.debug("ShoppingCart.checkInventory() - checking Inventory quantity of item: " + si.getID());
		Inventory inv = getInventoryItem(si.getID());
		
		/**
		 * Decrease the quantity of this inventory item.
		 * @param quantity The number to decrease the inventory by.
		 * @return The number of inventory items removed.
		 */
		int quantity=si.getQuantity();
		int minimumItems = inv.getMinThreshold();
			
		int amountToOrder = 0;
		Util.debug("ShoppingCartBean:checkInventory() - Decreasing inventory item " +inv.getInventoryId());
		int quantityNotFilled = 0;
		if (inv.getQuantity() < 1)
		{
			quantityNotFilled = quantity;
		}
		else if (inv.getQuantity() < quantity)
		{
			quantityNotFilled = quantity - inv.getQuantity();
		}
			
		// When quantity becomes < 0, this will be to determine the
		// quantity of unfilled orders due to insufficient stock.
		inv.setQuantity(inv.getQuantity() - quantity);
			
		//  Check to see if more inventory needs to be ordered from the supplier
		//  based on a set minimum Threshold
		if (inv.getQuantity() < minimumItems)
		{
			// Calculate the amount of stock to order from the supplier
			// to get the inventory up to the maximum.
			amountToOrder = quantityNotFilled;
			backOrder(inv, amountToOrder);
		}
			
		
	}
	
	/**
	 * Create a BackOrder of this inventory item.
	 * @param quantity The number of the inventory item to be backordered
	 */
	private void backOrder(Inventory inv, int amountToOrder)
	{
		BackOrder b=em.find(BackOrder.class, inv.getInventoryId());
		if (b == null) {
			//create a new backorder if none exists
			BackOrder newBO=new BackOrder(inv, amountToOrder);
			em.persist(newBO);
			em.flush();
			inv.setBackOrder(newBO);
		} else {
			//update the backorder with the new quantity			
			int quantity=b.getQuantity();
			quantity+=amountToOrder;			
			em.lock(b, LockModeType.WRITE);
			em.refresh(b);
			b.setQuantity(quantity);
			em.flush();
			inv.setBackOrder(b);
		}
	}
	/** 
	 * Method checkInventory.
	 * Check the inventory level all items in the shopping cart.
	 * Order additional inventory when necessary
	 *
	 * @param items
	 */
	public void checkInventory(Collection <ShoppingCartItem>items)
	{
		ShoppingCartItem si;
		for (Object o : items)
		{
			si = (ShoppingCartItem) o;
			checkInventory(si);			
		}
	}
	/**
	 * Get the contents of the shopping cart.
	 *
	 * @return The contents of the shopping cart.
	 */
	public ShoppingCartContents getCartContents()
	{
		ShoppingCartContents cartContents = new ShoppingCartContents();
		// Fill it with data.
		for (int i = 0; i < items.size(); i++)
		{
			cartContents.addItem((ShoppingCartItem) items.get(i));
		}
		return cartContents;
	}
	/**
	 * Create an order with contents of a shopping cart.
	 *
	 * @param customerID customer's ID
	 * @param billName billing name
	 * @param billAddr1 billing address line 1
	 * @param billAddr2 billing address line 2
	 * @param billCity billing address city
	 * @param billState billing address state
	 * @param billZip billing address zip code
	 * @param billPhone billing phone
	 * @param shipName shippng name
	 * @param shipAddr1 shippng address line 1
	 * @param shipAddr2 shippng address line 2
	 * @param shipCity shippng address city
	 * @param shipState shippng address state
	 * @param shipZip shippng address zip code
	 * @param shipPhone shippng phone
	 * @param creditCard credit card
	 * @param ccNum credit card number
	 * @param ccExpireMonth credit card expiration month
	 * @param ccExpireYear credit card expiration year
	 * @param cardHolder credit card holder name
	 * @param shippingMethod int of shipping method used
	 * @param items vector of StoreItems ordered
	 * @return OrderInfo
	 */
	public OrderInfo createOrder(
			String customerID,
			String billName,
			String billAddr1,
			String billAddr2,
			String billCity,
			String billState,
			String billZip,
			String billPhone,
			String shipName,
			String shipAddr1,
			String shipAddr2,
			String shipCity,
			String shipState,
			String shipZip,
			String shipPhone,
			String creditCard,
			String ccNum,
			String ccExpireMonth,
			String ccExpireYear,
			String cardHolder,
			int shippingMethod,
			Collection <ShoppingCartItem>items)
	{
		Order order = null;
		/*
		 try
		 {
		 OrderHome orderHome = (OrderHome) Util.getEJBLocalHome("java:comp/env/ejb/Order", OrderHome.class);
		 Util.debug("ShoppingCartBean.createOrder:  Creating Order");
		 order = orderHome.create(customerID, billName, billAddr1, billAddr2, billCity, billState, billZip, 
		 billPhone, shipName, shipAddr1, shipAddr2, shipCity, shipState, shipZip, shipPhone, creditCard, 
		 ccNum, ccExpireMonth, ccExpireYear, cardHolder, shippingMethod, items);
		 */
		Collection<OrderItem> orderitems = new ArrayList<OrderItem>();
		for (Object o : items) {
			ShoppingCartItem si = (ShoppingCartItem) o;
			Inventory inv = em.find(Inventory.class, si.getID());
			OrderItem oi = new OrderItem(inv);
			oi.setQuantity(si.getQuantity());
			orderitems.add(oi); 
		}
		Customer c = em.find(Customer.class, customerID);
		order = new Order(c, billName, billAddr1, billAddr2, billCity, billState, billZip, billPhone,
				shipName, shipAddr1, shipAddr2, shipCity, shipState, shipZip, shipPhone, creditCard,
				ccNum, ccExpireMonth, ccExpireYear, cardHolder, shippingMethod, orderitems);		
		em.persist(order);
		em.flush();  
		//store the order items
		for (OrderItem o : orderitems) {
			o.setOrder(order);
			o.updatePK();
			em.persist(o);
		}
		em.flush();
		
		OrderInfo orderInfo=new OrderInfo(order);
		/*
		 }
		 catch (CreateException e)
		 {
		 Util.debug("ShoppingCartBean(createOrder): Exception - " + e);
		 e.printStackTrace();
		 }
		 if (order != null)
		 {
		 */
		return orderInfo;
		/*
		 }
		 else
		 return null; */
	}

}
